@@ -55,6 +55,8 @@ static int irq_installed = 0;
 static char **sound_buffers = NULL;
 static char **sound_read_buffers = NULL;
 
+static DEFINE_SPINLOCK(cs4218_lock);
+
 /* Local copies of things we put in the control register.  Output
  * volume, like most codecs is really attenuation.
  */
@@ -597,7 +599,7 @@ static ssize_t cs4218_ctx_law(const u_char *userPtr, size_t userCount,
 	int hSpeed = sound.hard.speed, sSpeed = sound.soft.speed;
 	int utotal, ftotal;
 	int stereo = sound.soft.stereo;
- 
+
 	frameLeft >>= 2;
 	if (stereo)
 		userCount >>= 1;
@@ -1206,7 +1208,8 @@ static void CS_Play(void)
 	volatile cbd_t	*bdp;
 	volatile cpm8xx_t *cp;
 
-	save_flags(flags); cli();
+	/* Protect buffer */
+	spin_lock_irqsave(&cs4218_lock, flags);
 #if 0
 	if (awacs_beep_state) {
 		/* sound takes precedence over beeps */
@@ -1263,7 +1266,7 @@ static void CS_Play(void)
 
 		++sq.active;
 	}
-	restore_flags(flags);
+	spin_unlock_irqrestore(&cs4218_lock, flags);
 }
 
 
@@ -1275,7 +1278,8 @@ static void CS_Record(void)
 	if (read_sq.active)
 		return;
 
-	save_flags(flags); cli();
+	/* Protect buffer */
+	spin_lock_irqsave(&cs4218_lock, flags);
 
 	/* This is all we have to do......Just start it up.
 	*/
@@ -1284,7 +1288,7 @@ static void CS_Record(void)
 
 	read_sq.active = 1;
 
-        restore_flags(flags);
+        spin_unlock_irqrestore(&cs4218_lock, flags);
 }
 
 
@@ -1365,14 +1369,15 @@ static void cs_nosound(unsigned long xx)
 {
 	unsigned long flags;
 
-	save_flags(flags); cli();
+	/* not sure if this is needed, since hardware command is #if 0'd */
+	spin_lock_irqsave(&cs4218_lock, flags);
 	if (beep_playing) {
 #if 0
 		st_le16(&beep_dbdma_cmd->command, DBDMA_STOP);
 #endif
 		beep_playing = 0;
 	}
-	restore_flags(flags);
+	spin_unlock_irqrestore(&cs4218_lock, flags);
 }
 
 static struct timer_list beep_timer = TIMER_INITIALIZER(cs_nosound, 0, 0);
@@ -1401,21 +1406,22 @@ static void cs_mksound(unsigned int hz, unsigned int ticks)
 		return;
 #endif
 	}
-	save_flags(flags); cli();
+	/* lock while modifying beep_timer */
+	spin_lock_irqsave(&cs4218_lock, flags);
 	del_timer(&beep_timer);
 	if (ticks) {
 		beep_timer.expires = jiffies + ticks;
 		add_timer(&beep_timer);
 	}
 	if (beep_playing || sq.active || beep_buf == NULL) {
-		restore_flags(flags);
+		spin_unlock_irqrestore(&cs4218_lock, flags);
 		return;		/* too hard, sorry :-( */
 	}
 	beep_playing = 1;
 #if 0
 	st_le16(&beep_dbdma_cmd->command, OUTPUT_MORE + BR_ALWAYS);
 #endif
-	restore_flags(flags);
+	spin_unlock_irqrestore(&cs4218_lock, flags);
 
 	if (hz == beep_hz_cache && beep_volume == beep_volume_cache) {
 		nsamples = beep_nsamples_cache;
@@ -1442,7 +1448,7 @@ static void cs_mksound(unsigned int hz, unsigned int ticks)
 	st_le32(&beep_dbdma_cmd->phy_addr, virt_to_bus(beep_buf));
 	awacs_beep_state = 1;
 
-	save_flags(flags); cli();
+	spin_lock_irqsave(&cs4218_lock, flags);
 	if (beep_playing) {	/* i.e. haven't been terminated already */
 		out_le32(&awacs_txdma->control, (RUN|WAKE|FLUSH|PAUSE) << 16);
 		out_le32(&awacs->control,
@@ -1452,25 +1458,14 @@ static void cs_mksound(unsigned int hz, unsigned int ticks)
 		out_le32(&awacs_txdma->cmdptr, virt_to_bus(beep_dbdma_cmd));
 		out_le32(&awacs_txdma->control, RUN | (RUN << 16));
 	}
+	spin_unlock_irqrestore(&cs4218_lock, flags);
 #endif
-	restore_flags(flags);
-}
-
-static void CS_open(void)
-{
-	MOD_INC_USE_COUNT;
-}
-
-static void CS_release(void)
-{
-	MOD_DEC_USE_COUNT;
 }
 
 static MACHINE mach_cs4218 = {
+	.owner =	THIS_MODULE,
 	.name =		"HIOX CS4218",
 	.name2 =	"Built-in Sound",
-	.open =		CS_open,
-	.release =	CS_release,
 	.dma_alloc =	CS_Alloc,
 	.dma_free =	CS_Free,
 	.irqinit =	CS_IrqInit,
@@ -1629,16 +1624,14 @@ static ssize_t sound_copy_translate_read(const u_char *userPtr,
 
 static int mixer_open(struct inode *inode, struct file *file)
 {
-	MOD_INC_USE_COUNT;
 	mixer.busy = 1;
-	return 0;
+	return nonseekable_open(inode, file);
 }
 
 
 static int mixer_release(struct inode *inode, struct file *file)
 {
 	mixer.busy = 0;
-	MOD_DEC_USE_COUNT;
 	return 0;
 }
 
@@ -1656,8 +1649,8 @@ static int mixer_ioctl(struct inode *inode, struct file *file, u_int cmd,
 	switch (cmd) {
 		case SOUND_MIXER_INFO: {
 		    mixer_info info;
-		    strncpy(info.id, "CS4218_TDM", sizeof(info.id));
-		    strncpy(info.name, "CS4218_TDM", sizeof(info.name));
+		    strlcpy(info.id, "CS4218_TDM", sizeof(info.id));
+		    strlcpy(info.name, "CS4218_TDM", sizeof(info.name));
 		    info.name[sizeof(info.name)-1] = 0;
 		    info.modify_counter = mixer.modify_counter;
 		    if (copy_to_user((int *)arg, &info, sizeof(info)))
@@ -2050,7 +2043,6 @@ static int sq_open(struct inode *inode, struct file *file)
 {
 	int rc = 0;
 
-	MOD_INC_USE_COUNT;
 	if (file->f_mode & FMODE_WRITE) {
 		if (sq.busy) {
 			rc = -EBUSY;
@@ -2090,7 +2082,7 @@ static int sq_open(struct inode *inode, struct file *file)
 
 		read_sq_setup(numReadBufs,readbufSize<<10, sound_read_buffers);
 		read_sq.open_mode = file->f_mode;
-	}                                                                      
+	}
 
 	/* Start up the 4218 by:
 	 * Reset.
@@ -2109,17 +2101,17 @@ static int sq_open(struct inode *inode, struct file *file)
 	 */
 	cs4218_ctl_write(cs4218_control);
 
-	sound.minDev = MINOR(inode->i_rdev) & 0x0f;
+	sound.minDev = iminor(inode) & 0x0f;
 	sound.soft = sound.dsp;
 	sound.hard = sound.dsp;
 	sound_init();
-	if ((MINOR(inode->i_rdev) & 0x0f) == SND_DEV_AUDIO) {
+	if ((iminor(inode) & 0x0f) == SND_DEV_AUDIO) {
 		sound_set_speed(8000);
 		sound_set_stereo(0);
 		sound_set_format(AFMT_MU_LAW);
 	}
 
-	return 0;
+	return nonseekable_open(inode, file);
 
 err_out_nobusy:
 	if (file->f_mode & FMODE_WRITE) {
@@ -2131,7 +2123,6 @@ err_out_nobusy:
 		WAKE_UP(read_sq.open_queue);
 	}
 err_out:
-	MOD_DEC_USE_COUNT;
 	return rc;
 }
 
@@ -2183,7 +2174,6 @@ static int sq_release(struct inode *inode, struct file *file)
 
 	sq_release_read_buffers();
 	sq_release_buffers();
-	MOD_DEC_USE_COUNT;
 
 	if (file->f_mode & FMODE_READ) {
 		read_sq.busy = 0;
@@ -2372,7 +2362,6 @@ static int state_open(struct inode *inode, struct file *file)
 	if (state.busy)
 		return -EBUSY;
 
-	MOD_INC_USE_COUNT;
 	state.ptr = 0;
 	state.busy = 1;
 
@@ -2421,14 +2410,13 @@ static int state_open(struct inode *inode, struct file *file)
 	len += sprintf(buffer+len, "\tsq.active = %d sq.syncing = %d\n",
 		       sq.active, sq.syncing);
 	state.len = len;
-	return 0;
+	return nonseekable_open(inode, file);
 }
 
 
 static int state_release(struct inode *inode, struct file *file)
 {
 	state.busy = 0;
-	MOD_DEC_USE_COUNT;
 	return 0;
 }
 
@@ -2481,7 +2469,7 @@ static long long sound_lseek(struct file *file, long long offset, int orig)
 int __init tdm8xx_sound_init(void)
 {
 	int i, has_sound;
-	uint			dp_addr;
+	uint			dp_offset;
 	volatile uint		*sirp;
 	volatile cbd_t		*bdp;
 	volatile cpm8xx_t	*cp;
@@ -2502,7 +2490,7 @@ int __init tdm8xx_sound_init(void)
 	cp->cp_simode &= ~0x00000fff;
 
 	/* Enable common receive/transmit clock pins, use IDL format.
-	 * Sync on falling edge, transmit rising clock, recieve falling
+	 * Sync on falling edge, transmit rising clock, receive falling
 	 * clock, delay 1 bit on both Tx and Rx.  Common Tx/Rx clocks and
 	 * sync.
 	 * Connect SMC2 to TSA.
@@ -2543,14 +2531,14 @@ int __init tdm8xx_sound_init(void)
 	/* We need to allocate a transmit and receive buffer
 	 * descriptors from dual port ram.
 	 */
-	dp_addr = m8xx_cpm_dpalloc(sizeof(cbd_t) * numReadBufs);
+	dp_addr = cpm_dpalloc(sizeof(cbd_t) * numReadBufs, 8);
 
 	/* Set the physical address of the host memory
 	 * buffers in the buffer descriptors, and the
 	 * virtual address for us to work with.
 	 */
 	bdp = (cbd_t *)&cp->cp_dpmem[dp_addr];
-	up->smc_rbase = dp_addr;
+	up->smc_rbase = dp_offset;
 	rx_cur = rx_base = (cbd_t *)bdp;
 
 	for (i=0; i<(numReadBufs-1); i++) {
@@ -2565,10 +2553,10 @@ int __init tdm8xx_sound_init(void)
 
 	/* Now, do the same for the transmit buffers.
 	*/
-	dp_addr = m8xx_cpm_dpalloc(sizeof(cbd_t) * numBufs);
+	dp_offset = cpm_dpalloc(sizeof(cbd_t) * numBufs, 8);
 
 	bdp = (cbd_t *)&cp->cp_dpmem[dp_addr];
-	up->smc_tbase = dp_addr;
+	up->smc_tbase = dp_offset;
 	tx_cur = tx_base = (cbd_t *)bdp;
 
 	for (i=0; i<(numBufs-1); i++) {
@@ -2703,24 +2691,24 @@ void __init dmasound_setup(char *str, int *ints)
 	switch (ints[0]) {
 	case 3:
 		if ((ints[3] < 0) || (ints[3] > MAX_CATCH_RADIUS))
-			printk("dmasound_setup: illegal catch radius, using default = %d\n", catchRadius);
+			printk("dmasound_setup: invalid catch radius, using default = %d\n", catchRadius);
 		else
 			catchRadius = ints[3];
 		/* fall through */
 	case 2:
 		if (ints[1] < MIN_BUFFERS)
-			printk("dmasound_setup: illegal number of buffers, using default = %d\n", numBufs);
+			printk("dmasound_setup: invalid number of buffers, using default = %d\n", numBufs);
 		else
 			numBufs = ints[1];
 		if (ints[2] < MIN_BUFSIZE || ints[2] > MAX_BUFSIZE)
-			printk("dmasound_setup: illegal buffer size, using default = %d\n", bufSize);
+			printk("dmasound_setup: invalid buffer size, using default = %d\n", bufSize);
 		else
 			bufSize = ints[2];
 		break;
 	case 0:
 		break;
 	default:
-		printk("dmasound_setup: illegal number of arguments\n");
+		printk("dmasound_setup: invalid number of arguments\n");
 	}
 }
 
