@@ -6,31 +6,41 @@
  * arch/sh64/kernel/init_task.c
  *
  * Copyright (C) 2000, 2001  Paolo Alberelli
+ * Copyright (C) 2003  Paul Mundt
  *
  */
-
+#include <linux/rwsem.h>
 #include <linux/mm.h>
 #include <linux/sched.h>
-#include <linux/init.h>
+#include <linux/init_task.h>
+#include <linux/mqueue.h>
 
 #include <asm/uaccess.h>
 #include <asm/pgtable.h>
 
 static struct fs_struct init_fs = INIT_FS;
 static struct files_struct init_files = INIT_FILES;
-static struct signal_struct init_signals = INIT_SIGNALS;
+static struct signal_struct init_signals = INIT_SIGNALS(init_signals);
+static struct sighand_struct init_sighand = INIT_SIGHAND(init_sighand);
 struct mm_struct init_mm = INIT_MM(init_mm);
 
-
 struct pt_regs fake_swapper_regs;
 
 /*
- * Initial task structure.
+ * Initial thread structure.
  *
- * We need to make sure that this is INIT_TASK_SIZE-byte aligned due
+ * We need to make sure that this is THREAD_SIZE-byte aligned due
  * to the way process stacks are handled. This is done by having a
  * special "init_task" linker map entry..
  */
-union task_union init_task_union 
+union thread_union init_thread_union
 	__attribute__((__section__(".data.init_task"))) =
-		{ INIT_TASK(init_task_union.task) };
+		{ INIT_THREAD_INFO(init_task) };
+
+/*
+ * Initial task structure.
+ *
+ * All other task structs will be allocated on slabs in fork.c
+ */
+struct task_struct init_task = INIT_TASK(init_task);
+
